package com.qhtx.eta.Advice;

import com.google.common.collect.Maps;
import com.qhtx.eta.common.exception.ETAException;
import com.qhtx.eta.vo.Result;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import java.util.Map;

@RestControllerAdvice
public class IntegratorControllerAdvice {
    private static final Logger LOGGER = LoggerFactory.getLogger(IntegratorControllerAdvice.class);

    /**
     * 自定义方法参数校验异常处理器
     *
     * @param ex
     * @return
     */
    @ExceptionHandler(MethodArgumentNotValidException.class)
    @ResponseStatus(HttpStatus.BAD_REQUEST)
    @ResponseBody
    public Map<String, String> handleValidationExceptions(MethodArgumentNotValidException ex) {
        Map<String, String> errors = Maps.newHashMapWithExpectedSize(1);
        ex.getBindingResult().getAllErrors().forEach((error) -> {
            String fieldName = ((FieldError) error).getField();
            String errorMessage = error.getDefaultMessage();
            errors.put(fieldName, errorMessage);
        });
        return errors;
    }

    /**
     * 自定义业务异常处理器
     *
     * @param etaException
     * @return
     */
    @ExceptionHandler(ETAException.class)
    @ResponseStatus(HttpStatus.OK)
    @ResponseBody
    public Result exceptionHandler(ETAException etaException) {
        Result result = new Result();
        result.setCode(etaException.getErrorCode());
        result.setMessage(etaException.getErrorMessage());
        result.setSuccess(false);
        return result;
    }

    /**
     * 处理所有未捕获的异常
     *
     * @param ex
     * @return
     */
    @ExceptionHandler(Exception.class)
    @ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
    public Result handleAllExceptions(Exception ex) {
        LOGGER.error("Exception:", ex);
        Result result = new Result();
        result.setCode("10000");
        result.setMessage("未知异常");
        result.setSuccess(false);
        return result;
    }

}
