package com.qhtx.eta.common.entity;

import lombok.Data;
import org.apache.commons.lang3.StringUtils;

import java.util.HashMap;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 验签参数
 * Created by chenhan on 2024/04/24.
 */
@Data
public class SignatureParam {
    /**
     * 随机字符串
     */
    private String nonce;
    /**
     * 时间戳
     */
    private Long timestamp;

    /**
     * 应用ID
     */
    private String appid;

    private String secretKey;

    public SignatureParam(Builder builder) {
        this.nonce = builder.nonce;
        this.timestamp = builder.timestamp;
        this.appid = builder.appid;
        this.secretKey = builder.secretKey;
    }


    public static class Builder {
        /**
         * 随机字符串
         */
        private String nonce;
        /**
         * 时间戳
         */
        private Long timestamp;
        /**
         * 应用ID
         */
        private String appid;
        /**
         * 密钥
         */
        private String secretKey;

        /**
         * 设置随机字符串
         */
        public Builder nonce(String nonce) {
            this.nonce = nonce;
            return this;
        }

        /**
         * 设置时间戳
         */
        public Builder timestamp(Long timestamp) {
            this.timestamp = timestamp;
            return this;
        }

        /**
         * 设置应用ID
         */
        public Builder appid(String appid) {
            this.appid = appid;
            return this;
        }

        public Builder secretKey(String secretKey) {
            this.secretKey = secretKey;
            return this;
        }

        /**
         * 构建SignatureParam实例
         */
        public SignatureParam build() {
            // 可在此处添加必要的参数校验，如不能为空等
            return new SignatureParam(this);
        }
    }


    @Override
    public String toString() {
        Map<String, Object> resultMap = new HashMap<>(3);
        if (StringUtils.isNotBlank(this.appid)) {
            resultMap.put("appid", this.appid);
        }
        if (StringUtils.isNotBlank(this.nonce)) {
            resultMap.put("nonce", this.nonce);
        }
        if (this.timestamp != null) {
            resultMap.put("timestamp", this.timestamp);
        }
        return resultMap.entrySet().stream().map(entry -> entry.getKey() + "=" + entry.getValue()).sorted().collect(Collectors.joining("&"));
    }
}
