package com.qhtx.eta.domain.api;

import com.qhtx.eta.domain.enums.ETAInterfaceEnum;
import com.qhtx.eta.domain.api.http.response.inner.InnerResponse;
import com.qhtx.eta.domain.api.proxy.ApiParameterProxy;
import com.qhtx.eta.common.eunms.ErrorEnum;
import com.qhtx.eta.common.exception.ETAException;
import com.qhtx.eta.domain.api.http.ETAHttpClient;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.lang.reflect.InvocationTargetException;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * API 接口持有者
 */
@Component
@Slf4j
public class ApiServiceHolder {

    private static final Map<ETAInterfaceEnum, ApiParameterProxy> proxyMap = new ConcurrentHashMap<>();
    private ETAHttpClient etaHttpClient;

    public void registerClient(ETAHttpClient etaHttpClient) {
        this.etaHttpClient = etaHttpClient;
    }

    public void register(ETAInterfaceEnum etaInterfaceEnum) {
        try {
            Class<? extends ApiParameterProxy> proxy = etaInterfaceEnum.getProxy();
            proxyMap.put(etaInterfaceEnum, proxy.getConstructor().newInstance());
        } catch (NoSuchMethodException | InstantiationException | IllegalAccessException |
                 InvocationTargetException e) {
            log.warn("注册ETA代理失败{}", e.getMessage());
            throw new ETAException(ErrorEnum.PROXY_REGISTER_ERROR);
        }

    }

    /**
     * API接口调用
     *
     * @return Response
     */
    public List<?> runApi(ETAInterfaceEnum etaInterfaceEnum) {
        return runApi(etaInterfaceEnum, null);
    }

    public List<?> runApi(ETAInterfaceEnum etaInterfaceEnum, Object param) {
        if (!etaInterfaceEnum.getEnable()) {
            throw new ETAException(ErrorEnum.API_NOT_FOUND);
        }
        if (etaInterfaceEnum.getResponse() == null) {
            throw new ETAException(ErrorEnum.API_RESPONSE_CONFIG_ERROR);
        }
        ApiParameterProxy<?> apiParameterProxy = proxyMap.get(etaInterfaceEnum);
        Map<String, Object> params = null;
        if (apiParameterProxy != null) {
            params = apiParameterProxy.parameters(param);
        }
        InnerResponse response = etaHttpClient.send(etaInterfaceEnum, params);
        if (isSuccess(response)) {
            return response.getData();
        }
        log.warn("调用接口失败:{}", response.getErrorMsg());
        return null;
    }

    private boolean isSuccess(InnerResponse response) {
        return response != null && response.getSuccess();
    }
}
