package com.qhtx.eta.domain.service.impl;

import com.qhtx.eta.common.constant.ETAConstants;
import com.qhtx.eta.common.eunms.ErrorEnum;
import com.qhtx.eta.common.exception.ETAException;
import com.qhtx.eta.domain.convert.DWIndexDTOConverter;
import com.qhtx.eta.domain.convert.DWIndexDataDTOConverter;
import com.qhtx.eta.domain.entity.DWIndexDTO;
import com.qhtx.eta.domain.entity.DWIndexDataDTO;
import com.qhtx.eta.domain.service.ETAPushDataService;
import com.qhtx.eta.infra.entity.dw.TDampDwIndexData;
import com.qhtx.eta.infra.entity.dw.TEtaIndexPushTask;
import com.qhtx.eta.infra.service.TDampDwIndexDataService;
import com.qhtx.eta.infra.service.TEtaIndexPushTaskService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.stream.Collectors;

@Service
@Slf4j
public class ETAPushDataServiceImpl implements ETAPushDataService {

    @Resource
    private TEtaIndexPushTaskService tEtaIndexPushTaskService;

    @Resource
    private TDampDwIndexDataService tDampDwIndexDataService;
    @Resource(name = "IndexDataThreadPool")
    private ThreadPoolExecutor indexDataThreadPool;

    @Override
    public List<DWIndexDTO> getPushIndexTaskList() {
        return DWIndexDTOConverter.INSTANCE.convertToDTOList(tEtaIndexPushTaskService.getAllTask());
    }

    @Override
    public List<DWIndexDataDTO> getUpdateDataList(String indexCode, String beginDate) {
        int total = tDampDwIndexDataService.countByIndexCodeWithDate(indexCode, beginDate);
        List<TDampDwIndexData> allResults;
        //分块取值
        if (total > ETAConstants.ETA_PUSH_DATA_LIMIT) {
            int chunk = (total + ETAConstants.ETA_PUSH_DATA_LIMIT - 1) / ETAConstants.ETA_PUSH_DATA_LIMIT;
            List<CompletableFuture<List<TDampDwIndexData>>> futures = new ArrayList<>();
            for (int i = 0; i < chunk; i++) {
                int pars = i;
                CompletableFuture<List<TDampDwIndexData>> future = CompletableFuture.supplyAsync(() -> {
                    try {
                        return tDampDwIndexDataService.queryByDate(indexCode, beginDate, pars * ETAConstants.ETA_PUSH_DATA_LIMIT, ETAConstants.ETA_PUSH_DATA_LIMIT);
                    } catch (Exception e) {
                        log.error("获取指标数据失败", e);
                        throw new ETAException(ErrorEnum.DATASOURCE_EXECUTE_ERROR);
                    }
                }, indexDataThreadPool);
                futures.add(future);
            }
            // 等待所有任务完成并合并结果
            allResults = futures.stream().map(CompletableFuture::join).flatMap(List::stream).collect(Collectors.toList());
        } else {
            allResults = tDampDwIndexDataService.queryByDate(indexCode, beginDate, 0, total);
        }
        return allResults.stream().map(DWIndexDataDTOConverter.INSTANCE::convertToDTO).collect(Collectors.toList());
    }

    @Override
    public TEtaIndexPushTask getETAPushTaskByIndexCode(String indexCode) {
        return tEtaIndexPushTaskService.getByIndexCode(indexCode);
    }

    @Override
    public void updateTaskByIndexCode(TEtaIndexPushTask tEtaIndexPushTask) {
        tEtaIndexPushTaskService.update(tEtaIndexPushTask);
    }
}
